#include <Arduino.h>
#include "wiring.h"
#include "timers.h"

/** State of the dice automaton. It is either idle, waiting for the user to
    press the switch, or rolling until the final value of the dice pops.
    Once the final value is there, the automaton returns in the idle state. */
enum automaton_state_t { AS_Idle, AS_Rolling } ;
volatile enum automaton_state_t automaton_state = AS_Idle ;


/** The current dice value. Changed under interrupt by the timer handler. */
volatile uint16_t dice_val ;


/** Indirection table giving for each dice value - 1 the leds to light.
    If a led must be lighted, then the bit corresponding to its pin is
    1, else it is 0.
    This works as long as the used pins are between 0 and 7 included. Since in
    our case they are between 1 and 7, it's all good. */
unsigned char leds_encoding[6] = {
  /* Value 1. */
  (1 << PIN_LED_CENTER),
  /* Value 2. */
  (1 << PIN_LED_BOT_LEFT) | (1 << PIN_LED_TOP_RIGHT),
  /* Value 3. */
  (1 << PIN_LED_BOT_LEFT) | (1 << PIN_LED_CENTER) | (1 << PIN_LED_TOP_RIGHT),
  /* Value 4. */
  (1 << PIN_LED_TOP_LEFT) | (1 << PIN_LED_BOT_LEFT) |
  (1 << PIN_LED_TOP_RIGHT) | (1 << PIN_LED_BOT_RIGHT),
  /* Value 5. */
  (1 << PIN_LED_TOP_LEFT) | (1 << PIN_LED_BOT_LEFT) |
  (1 << PIN_LED_CENTER) |
  (1 << PIN_LED_TOP_RIGHT) | (1 << PIN_LED_BOT_RIGHT),
  /* Value 6. */
  (1 << PIN_LED_TOP_LEFT) | (1 << PIN_LED_MID_LEFT) | (1 << PIN_LED_BOT_LEFT) |
  (1 << PIN_LED_TOP_RIGHT) | (1 << PIN_LED_MID_RIGHT) | (1 << PIN_LED_BOT_RIGHT)
} ;


/** "Print" a value on the leds. This function uses the indirection table
    above to fing the leds to turn on and off. */
void print_dice (uint16_t val)
{
  uint16_t i ;
  unsigned char mask = 0x1 ;
  unsigned char code ;

  /* Just prevent wrong values from reading outside of the indirection table
     or try to change the state of wrong pins. */
  if ((val < 1) || (val > 6)) return ;
  code = leds_encoding[val -1] ;
  for (i = 0; i < 8; i++) {
    if (mask & code) digitalWrite (i, HIGH) ;
    else digitalWrite (i, LOW) ;
    mask = mask << 1 ;
  }
}


/* Values to initialize the timers. */
#define TIMER_CHANNEL (0)
#define INITIAL_PERIOD (0.05)                                     /* Seconds. */
#define INITIAL_FREQ (1.0 / INITIAL_PERIOD)                          /* Hz. */
#define INITIAL_TIMER_128_RC ((VARIANT_MCK) / 128 / (INITIAL_FREQ)) /* Ticks. */


volatile uint32_t timer_restart_value ;

/** Interrupt handler for the timer. */
void TC3_Handler ()
{
  /* Increase the waiting time to simulate the dice turning slower. */
  timer_restart_value += 12000 ;
  /* If duration is above a certain value (tweak it ^_^) then we consider
     that the dice is stopped. */
  if (timer_restart_value > (INITIAL_TIMER_128_RC * 7)) {
    /* Kill the timer and switch the automaton in idle state. */
     stop_timer_TC1 (TIMER_CHANNEL) ;
     automaton_state = AS_Idle ;
  }
  else {
    /* Get a random new value. We could be closer from the real model by
       forbidding to get the opposite "face" of the current. In effect,
       at each step, the dice can go forward/backward/left/right. Not
       directly flip of face. But ... is it so important ^_^ ? */
    dice_val = 1 + rand () % 6 ;
    /* Re-initialise the count value. */
    TC_SetRC (TC1, TIMER_CHANNEL, timer_restart_value) ;
    /* Get the status to clear it and allow the interrupt to fire again. */
    TC_GetStatus (TC1, TIMER_CHANNEL) ;
  }
}


void setup ()
{
  /* Set the pins modes. */
  pinMode (PIN_SWITCH, INPUT) ;
  pinMode (PIN_LED_TOP_LEFT, OUTPUT) ;
  pinMode (PIN_LED_MID_LEFT, OUTPUT) ;
  pinMode (PIN_LED_BOT_LEFT, OUTPUT) ;
  pinMode (PIN_LED_CENTER, OUTPUT) ;
  pinMode (PIN_LED_TOP_RIGHT, OUTPUT) ;
  pinMode (PIN_LED_MID_RIGHT, OUTPUT) ;
  pinMode (PIN_LED_BOT_RIGHT, OUTPUT) ;
  /* Initial value between 1 and 6 both included. */
  dice_val = 1 + rand () % 6 ;
  print_dice (dice_val) ;
}


void loop ()
{
  switch (automaton_state) {
  case AS_Idle:
    /* Just do simple: no need to use an interrupt. No need to filter the
       glitches since the automaton will go into another state and this
       part of code won't be executed for a while, hence ignoring glitches. */
    if (digitalRead (PIN_SWITCH) == HIGH) {
      automaton_state = AS_Rolling ;
      timer_restart_value = INITIAL_TIMER_128_RC ;
      start_timer_TC1 (timer_restart_value, TIMER_CHANNEL) ;
    }
    break ;

  case AS_Rolling:
    /* Simply print the dice... The timer handler will change the automaton
       state once the dice will be considered stopped.
    Indeed, we have in fact an automaton with ... 2 states ... a simple boolean
    test ^_^ */
    print_dice (dice_val) ;
    break ;
  }
}
